@@ -14,22 +14,23 @@ use MooseX::Params::Validate ();
 
 use Params::Registry::Template;
 use Params::Registry::Instance;
-use Params::Registry::Error;
 
 use URI;
 use URI::QueryParam;
 
+with 'Throwable';
+
 =head1 NAME
 
 Params::Registry - Housekeeping for sets of named parameters
 
 =head1 VERSION
 
-Version 0.09_05
+Version 0.01
 
 =cut
 
-our $VERSION = '0.09_05';
+our $VERSION = '0.01';
 
 =head1 SYNOPSIS
 
@@ -173,17 +174,14 @@ around BUILDARGS => sub {
             $map{$name} = $use;
         }
         # TODO: throw a proper error on duplicate key
-        Params::Registry::Error->throw
-              ("Parameter $name already exists") if exists $p{params}{$name};
-
+        die if exists $p{params}{$name};
         $p{params}{$name} = $entry;
     }
 
     # second pass to stitch the reused parameters together
     while (my ($k, $v) = each %map) {
         # TODO throw a proper error if the target isn't found
-        my $p = $p{params}{$v} or Params::Registry::Error->throw
-            ("Tried to resolve $v for reuse but couldn't find it");
+        my $p = $p{params}{$v} or die;
 
         # overwrite with any new data
         $p{params}{$k} = {%$p, %{$p{params}{$k}}};
@@ -205,9 +203,7 @@ sub BUILD {
         my %t = %{$p->{$k}};
         my $x = $p->{$k} = Params::Registry::Template->new
             (%t, registry => $self);
-        if ($x->_consdep > 0) {
-            # shortcut because only parameters with dependencies will
-            # have a rank higher than zero
+        if ($x->consumes > 0) {
             $rank{$k} = 1;
             push @stack, $k;
         }
@@ -218,15 +214,15 @@ sub BUILD {
 
     # construct a rank tree
 
-    #my %seen;
+    my %seen;
     while (my $x = shift @stack) {
         #my %c = map { $_ => 1 } $p->{$x}->consumes;
         my $match = 0;
-        for my $c ($p->{$x}->_consdep) {
+        for my $c ($p->{$x}->consumes) {
             $match = 1 if $rank{$x} == $rank{$c};
             # XXX will this actually catch all cycles?
-            Params::Registry::Error->throw
-                  ("Cycle detected between $x and $c") if $rank{$x} < $rank{$c};
+            Carp::croak("Cycle detected between $x and $c")
+                  if $rank{$x} < $rank{$c};
         }
 
         if ($match) {
@@ -351,7 +347,7 @@ sub process {
             $obj = $_[0];
         }
         else {
-            Params::Registry::Error->throw
+            $self->throw
                 ('If the argument is a ref, it must be a URI or a HASH ref');
         }
     }
@@ -365,13 +361,12 @@ sub process {
         $obj = \%x;
     }
     else {
-        Params::Registry::Error->throw
-              ('Check your inputs to Params::Registry::process');
+        $self->throw('Check your inputs to Params::Registry::process');
     }
 
     my $instance = Params::Registry::Instance->new(registry => $self);
 
-    $instance->set($obj, -defaults => 1, -force => 1);
+    $instance->set($obj, -defaults => 1);
 }
 
 =head2 template $KEY
@@ -436,24 +431,11 @@ L<http://search.cpan.org/dist/Params-Registry/>
 
 =back
 
+
 =head1 SEE ALSO
 
-=over 4
-
-=item
-
-L<Params::Registry::Instance>
-
-=item
-
-L<Params::Registry::Template>
-
-=item
-
 L<Params::Validate>
 
-=back
-
 =head1 LICENSE AND COPYRIGHT
 
 Copyright 2013 Dorian Taylor.
