@@ -9,9 +9,8 @@ use namespace::autoclean;
 
 use Params::Registry::Types qw(Type Dependency Format);
 use MooseX::Types::Moose    qw(Maybe Bool Int Str ArrayRef CodeRef);
-use Try::Tiny;
 
-use Params::Registry::Error;
+with 'Throwable';
 
 =head1 NAME
 
@@ -19,11 +18,11 @@ Params::Registry::Template - Template class for an individual parameter
 
 =head1 VERSION
 
-Version 0.04
+Version 0.01
 
 =cut
 
-our $VERSION = '0.04';
+our $VERSION = '0.01';
 
 =head1 SYNOPSIS
 
@@ -125,6 +124,7 @@ be specified with coercions that expect C<ArrayRef>, like so:
     },
     # ...
 
+
 =cut
 
 has composite => (
@@ -159,25 +159,18 @@ accompany this one.
 # I know it says ARRAY but the value is more useful as hash keys, so
 # these two attributes get coerced into hashrefs.
 
-has _depends => (
+has depends => (
     is      => 'ro',
     isa     => Dependency,
     traits  => [qw(Hash)],
     coerce  => 1,
     lazy    => 1,
-    init_arg => 'depends',
-    default => sub { Params::Registry::Types::ixhash_ref() },
+    default => sub { {} },
     handles => {
-        depends    => 'keys',
         depends_on => 'get',
     },
 );
 
-# # XXX HOLY SHIT TRAITS ARE SLOW
-# sub depends {
-#     keys %{$_[0]->_depends};
-# }
-
 =item conflicts
 
 An C<ARRAY> reference containing a list of parameters which I<must
@@ -192,7 +185,7 @@ has _conflicts => (
     coerce   => 1,
     lazy     => 1,
     init_arg => 'conflicts',
-    default  => sub { Params::Registry::Types::ixhash_ref() },
+    default  => sub { {} },
     handles  => {
         conflicts      => 'keys',
         conflicts_with => 'get',
@@ -201,11 +194,6 @@ has _conflicts => (
     },
 );
 
-# # XXX HOLY SHIT TRAITS ARE SLOW
-# sub conflicts {
-#     keys %{$_[0]->_conflicts};
-# }
-
 =item consumes
 
 For cascading parameters, an C<ARRAY> reference containing a list of
@@ -218,119 +206,31 @@ in the input at the same time.
 
 has _consumes => (
     is       => 'ro',
-    isa      => Dependency,
-    traits   => [qw(Hash)],
-    coerce   => 1,
+    isa      => ArrayRef[Str],
+    traits   => [qw(Array)],
     lazy     => 1,
     init_arg => 'consumes',
-    default  => sub { Params::Registry::Types::ixhash_ref() },
+    default  => sub { [] },
     handles  => {
-        consumes  => 'keys',
+        consumes => 'elements',
     },
 );
 
-# # XXX HOLY SHIT TRAITS ARE SLOW
-# sub consumes {
-#     keys %{$_[0]->_consumes};
-# }
+=item consumer
 
-has __consdep => (
-    is => 'ro',
-    isa => ArrayRef,
-    traits => [qw(Array)],
-    lazy    => 1,
-    default => sub { [ $_[0]->__UGH_CONSDEP ] },
-    handles => {
-        _consdep => 'elements',
-    },
-);
+For cascading parameters, a C<CODE> reference to operate on the
+consumed parameters in order to produce the desired I<atomic> value.
+To produce a I<composite> parameter value from multiple existing
+I<values>, define a coercion from C<ArrayRef> to the type supplied
+to the L</composite> property.
 
-# this thing merges 'consumes' and 'depends' together, in order
-sub __UGH_CONSDEP {
-    my $self = shift;
+The default consumer function, therefore, simply returns an C<ARRAY>
+reference that collates the values from the parameters defined in
+the L</consumes> property.
 
-    my $c = $self->_consumes;
-    $c = tied %$c;
-
-    my @out = $c->Keys;
-    my %c = map { $_ => 1 } @out;
-
-    # tack this on but only if there is a preprocessor present
-    if ($self->preproc) {
-        my $d = $self->_depends;
-        $d = tied %$d;
-        #my $c = $self->_consumes;
-        # ordered union of 'consumes' and 'depends'
-        push @out, grep { !$c{$_} } $d->Keys;
-    }
-
-    @out;
-}
-
-# change to 'preprocessor'
-
-# the purpose of the preprocessor is to coalesce values from multiple
-# parameters before handing them off to the template processor
-
-# these include the columns listed under 'depends' and 'consumes', the
-# difference between the two being that the former remain in the
-# resulting master data structure while the latter are removed.
-
-# the preprocessor function should therefore expect a list of array
-# refs: (should it? the dependencies will already have been processed)
-
-# change behaviour of 'depends' so that it can be cyclic *unless*
-# there is a preprocessor defined
-
-# this means the default has to be undef, so any code that uses the
-# default value has to be changed
-
-=item preproc
-
-Supply a C<CODE> reference to a function which coalesces values from
-the parameter in context (which may be empty) with other parameters
-specified by L</consumes> and L</depends>. The function is expected to
-return a result which can be handled by L</process>: either the
-appropriate L</composite> type (resulting in a no-op) or a list of
-valid primitives. The function is handed the following arguments:
-
-=over 4
-
-=item C<$self>
-
-The L<Params::Registry::Template> instance, to give the function
-(really a pseudo-method) access to its members.
-
-=item current raw value
-
-This will be an ARRAY reference containing zero or more elements, I<as
-supplied> to the input. It will B<not> be processed.
-
-=item other parameters
-
-All subsequent arguments to the L</preproc> function will represent
-the set union of L</consumes> and L</depends>. It will follow the
-sequence of keys specified in L</consumes> followed by the sequence in
-L</depends> B<minus> those which already appear in L</consumes>.
-
-It is important to note that I<these values will already have been
-processed>, so they will be whatever (potentially L</composite>) type
-you specify. Make sure you author this function with this expectation.
-
-=back
-
-The result(s) of L</preproc> will be collected into an array and fed
-into L</process>. Use L</depends> rather than L</consumes> to supply
-other parameters without removing them from the resulting structure.
-Note that when L</depends> is used in conjunction with L</preproc>,
-the dependencies I<must> be acyclic.
-
-L</preproc> is called either just before L</process> over supplied
-data, or in lieu of it.
-
-Here is an example of L</preproc> used to compose a set of parameters
-containing integers (e.g., from a legacy HTML form) into a L<DateTime>
-object:
+Once again, this functionality exists primarily for the purpose of
+interfacing with HTML forms that lack the latest features. Consider
+the following example:
 
     # ...
     {
@@ -363,106 +263,35 @@ object:
         consumes => [qw(year month day)],
 
         # and this is how it will happen:
-        preproc => sub {
-            my (undef, undef, $y, $m, $d) = @_;
+        consumer => sub {
             DateTime->new(
-                year  => $y,
-                month => $m,
-                day   => $d,
+                year  => $_[0],
+                month => $_[1],
+                day   => $_[2],
             );
         },
     },
     # ...
 
+Here, we may have a form which contains a C<date> field for the newest
+browsers that support the new form control, or otherwise generated via
+JavaScript. As a fallback mechanism (e.g. for an older browser, robot,
+or paranoid person), form fields for the C<year>, C<month>, and C<day>
+can also be specified in the markup, and used to generate C<date>.
+
 =cut
 
-has preproc => (
+sub _default_consume {
+    [@_];
+}
+
+has consumer => (
     is      => 'ro',
     isa     => CodeRef,
+    lazy    => 1,
+    default => sub { \&_default_consume },
 );
 
-# =item prefmt
-#
-# This element is the dual to L</preproc>.
-#
-# =cut
-
-# =item consumer
-
-# For cascading parameters, a C<CODE> reference to operate on the
-# consumed parameters in order to produce the desired I<atomic> value.
-# To produce a I<composite> parameter value from multiple existing
-# I<values>, define a coercion from C<ArrayRef> to the type supplied
-# to the L</composite> property.
-
-# The default consumer function, therefore, simply returns an C<ARRAY>
-# reference that collates the values from the parameters defined in
-# the L</consumes> property.
-
-# Once again, this functionality exists primarily for the purpose of
-# interfacing with HTML forms that lack the latest features. Consider
-# the following example:
-
-#     # ...
-#     {
-#         name => 'year',
-#         type => 'Int',
-#         max  => 1,
-#     },
-#     {
-#         name => 'month',
-#         type => 'Int',
-#         max  => 1,
-#     },
-#     {
-#         name => 'day',
-#         type => 'Int',
-#         max  => 1,
-#     },
-#     {
-#         name => 'date',
-
-#         # this would be defined elsewhere with coercion from a
-#         # string that matches 'YYYY-MM-DD', for direct input.
-#         type => 'MyDateTimeType',
-
-#         # we don't want multiple values for this parameter.
-#         max  => 1,
-
-#         # in lieu of being explicitly defined in the input, this
-#         # parameter will be constructed from the following:
-#         consumes => [qw(year month day)],
-
-#         # and this is how it will happen:
-#         consumer => sub {
-#             DateTime->new(
-#                 year  => $_[0],
-#                 month => $_[1],
-#                 day   => $_[2],
-#             );
-#         },
-#     },
-#     # ...
-
-# Here, we may have a form which contains a C<date> field for the newest
-# browsers that support the new form control, or otherwise generated via
-# JavaScript. As a fallback mechanism (e.g. for an older browser, robot,
-# or paranoid person), form fields for the C<year>, C<month>, and C<day>
-# can also be specified in the markup, and used to generate C<date>.
-
-# =cut
-
-# sub _default_consume {
-#     [@_];
-# }
-
-# has consumer => (
-#     is      => 'ro',
-#     isa     => CodeRef,
-#     lazy    => 1,
-#     default => sub { \&_default_consume },
-# );
-
 # =item cardinality
 
 # Either a scalar depicting an exact count, or a two-element C<ARRAY>
@@ -560,20 +389,15 @@ This C<default> value is passed through to the application only if the
 parameter in question is either missing or empty (if C<empty> is
 set). Likewise if the final translation of the input value matches the
 default, it will not show up in the canonical serialization. Like
-L<Moose>, is expected to be a C<CODE> reference. The subroutine takes
-two arguments: this template object, and the
-L<Params::Registry::Instance> object.
+L<Moose>, is expected to be a C<CODE> reference.
 
-    default => sub {
-        my ($t, $i) = @_;
-        # do something...
-    },
+    default => sub { 1 },
 
 =cut
 
 has default => (
     is  => 'ro',
-    isa => CodeRef|Str,
+    isa => CodeRef,
 );
 
 =item universe
@@ -591,10 +415,9 @@ If you specify a universe, you I<must> also specify a L</complement>.
 
 =cut
 
-has _universe => (
-    is       => 'ro',
-    isa      => CodeRef,
-    init_arg => 'universe',
+has universe => (
+    is  => 'ro',
+    isa => CodeRef,
 );
 
 # this is the cache for whatever gets generated by the universe function
@@ -602,10 +425,6 @@ has _unicache => (
     is  => 'rw',
 );
 
-sub universe {
-    $_[0]->_unicache;
-}
-
 =item complement
 
 For L</Set> and L</Range> parameters, this C<CODE> reference will need
@@ -633,11 +452,7 @@ has _complement => (
 sub complement {
     my ($self, $set) = @_;
     if (my $c = $self->_complement) {
-        try {
-            $c->($set, $self->_unicache);
-        } catch {
-            Params::Registry::Error->throw("Could not execute complement: $_");
-        };
+        $c->($set, $self->_unicache);
     }
 }
 
@@ -645,47 +460,35 @@ sub has_complement {
     return !!shift->_complement;
 }
 
-# XXX what is this bullshit about an unblessed hashref?
-
-# ... C<ARRAY> reference of scalars, or an I<unblessed> C<HASH>
-# reference containing valid parameter keys to either scalars or
-# C<ARRAY> references of scalars. In the case the subroutine returns a
-# C<HASH> reference, the registry will replace the parameter in
-# context with the parameters supplied, effectively performing the
-# inverse of a composite type coercion function.
-
 =item unwind
 
-Specify a C<CODE> reference to a subroutine which will turn the object
-into either a scalar or an C<ARRAY> reference of scalars. To encourage
-code reuse, this function is applied before L</reverse> despite the
-obvious ability to reverse the resulting list within the function.
+For composite object parameters, specify a C<CODE> reference to a
+subroutine which will turn the object into either a scalar, an
+C<ARRAY> reference of scalars, or an I<unblessed> C<HASH> reference
+containing valid parameter keys to either scalars or C<ARRAY>
+references of scalars. In the case the subroutine returns a C<HASH>
+reference, the registry will replace the parameter in context with the
+parameters supplied, effectively performing the inverse of a composite
+type coercion function. To encourage code reuse, this function is
+applied before L</reverse> despite the ability to reverse the
+resulting list in the function.
 
 The first argument to the subroutine is the template object itself,
-and the second is the value to be unwound. Subsequent arguments are
-the values of the parameters specified in L</depends>, if present.
-
-    sub my_unwind {
-        my ($self, $obj, @depends) = @_;
-        # ...
-    }
-
-If you don't need any state data from the template, consider the
-following idiom:
+and the second is the value to be unwound. If you don't need any state
+data from the template, consider the following idiom:
 
     {
         # ...
-        # assuming the object is a Set::Scalar
+        # assuming Set::Scalar
         unwind => sub { [sort $_[1]->elements] },
         # ...
     }
 
-For multi-valued parameters, an optional second return value can be
-used to indicate that the special
-L<complement|Params::Registry/complement> parameter should be set for
-this parameter. This is applicable, for instance, to the complement of
-a range, which would otherwise be impossible to serialize into a
-string.
+An optional second return value can be used to indicate that the
+special L<complement|Params::Registry/complement> parameter should be
+set for this parameter. This is applicable, for instance, to the
+complement of a range, which would otherwise be impossible to
+serialize into a string.
 
 =cut
 
@@ -722,104 +525,76 @@ sub BUILD {
 
 =back
 
-=head2 process @VALS
+=head2 process
 
-Validate a list of individual parameter values and (optionally)
-construct a L</composite> value.
+Validate a set of individual parameter values and (optionally)
+construct a composite value.
 
 =cut
 
 sub process {
-    my ($self, @in) = @_;
+    my ($self, @values) = @_;
 
-    my $t  = $self->type;
-    # XXX get rid of AUTOLOAD garbage
-    $t = $t->__type_constraint if ref $t eq 'MooseX::Types::TypeDecorator';
-    my $e  = $self->empty;
-    my $ac = $t->coercion;
-
-    # filter input
-    my @out;
-    for my $v (@in) {
-        # deal with undef/empty string
-        if (!defined $v or $v eq '') {
-            # do not append to @out unless 'empty' is set
-            next unless $e;
-
-            # normalize to undef
-            undef $v;
-        }
-
-        if (defined $v) {
-            if ($ac) {
-                # coerce atomic type
-                try {
-                    my $tmp = $ac->coerce($v);
-                    $v = $tmp;
-                } catch {
-                    my $err = $_;
-                    Params::Registry::Error::Syntax->throw(
-                        value   => $v,
-                        message => $err,
-                    );
-                };
-            }
-            else {
-                # check resulting value
-                Params::Registry::Error::Syntax->throw(
-                    value   => $v,
-                    message => "Value '$v' is not a $t") unless $t->check($v);
-            }
-        }
-
-        push @out, $v;
-    }
+    my $t = $self->type;
 
     # deal with cardinality
-    if (my $max = $self->max) {
-        # force scalar
-        if ($max == 1) {
-            return unless @out; # this will be empty
-            return $out[0];
+    my $max = $self->max;
+    if (defined $max and @values > $max) {
+        if ($self->shift) {
+            splice @values, -$max, $max;
+        }
+        else {
+            splice @values, 0, $max;
         }
-
-        # force cardinality
-        splice @out, ($self->shift ? -$max : 0), $max if @out > $max;
     }
 
-    # coerce to composite
-    if (my $comp = $self->composite) {
-        # XXX again get rid of AUTOLOAD
-        $comp = $comp->__type_constraint
-            if ref $comp eq 'MooseX::Types::TypeDecorator';
+    # coerce atomic type
+    my $e  = $self->empty;
+    my $ac = $t->coercion;
+    for my $i (0..$#values) {
+
+        if ($e && (!defined $values[$i] or $values[$i] eq '')) {
+            undef $values[$i];
+            next;
+        }
+
+        if ($ac) {
+            # coerce
+            #warn defined $values[$i];
+            $values[$i] = $ac->coerce($values[$i]) if defined $values[$i];
+        }
+
+        # XXX proper error
+        if (defined $values[$i]) {
+            Carp::croak('lol fail') unless $t->check($values[$i]);
+        }
+    }
+
+    if (my $c = $self->composite) {
         # try to coerce into composite
-        if (my $cc = $comp->coercion) {
-            #warn "lol $c";
-            return $cc->coerce(\@out);
+        if (my $cc = $c->coercion) {
+            warn "lol $c";
+            return $cc->coerce(\@values);
         }
     }
 
-    # otherwise return list of values
-    return wantarray ? @out : \@out;
+    return $values[0] if defined $max && $max == 1;
+
+    return wantarray ? @values : \@values;
 }
 
 
-=head2 unprocess $OBJ, @REST
+=head2 unprocess
 
-Applies L</unwind> to C<$OBJ> to get an C<ARRAY> reference, then
-L</format> over each of the elements to get strings. In list context
-it will also return the flag from L</unwind> indicating that the
-L<complement|Params::Registry/complement> parameter should be set.
-
-This method is called by L<Params::Registry::Instance/as_string> and
-others to produce content which is amenable to serialization. As what
-happens there, the content of C<@REST> should be the values of the
-parameters specified in L</depends>.
+Apply L</unwind> to get an arrayref, then L</format> to get strings.
+In list context it will also return the flag from L</unwind>
+indicating that the L<complement|Params::Registry/complement>
+parameter should be set.
 
 =cut
 
 sub unprocess {
-    my ($self, $obj, @rest) = @_;
+    my ($self, $obj) = @_;
 
     # take care of empty property
     unless (defined $obj) {
@@ -834,28 +609,17 @@ sub unprocess {
     # i dunno, should we check these types on the way out?
 
     my $complement;
-    if (defined $obj and my $u = $self->unwind) {
-        try {
-            ($obj, $complement) = $u->($self, $obj, @rest);
-        } catch {
-            Params::Registry::Error->throw("Could not execute unwind: $_");
-        };
+    if ($self->composite) {
+        if (my $u = $self->unwind) {
+            ($obj, $complement) = $u->($self, $obj);
+        }
     }
 
-    if (defined $obj) {
-        $obj = [$obj] unless ref $obj eq 'ARRAY';
-
-    }
-    else {
-        $obj = [];
-    }
-
-    # prune output again
-    @$obj = grep { defined $_ } @$obj unless $self->empty;
+    $obj = [$obj] unless ref $obj eq 'ARRAY';
 
     # format values
     my $fmt = $self->format;
-    # XXX this should really be done once
+    # XXX this should really be done once 
     #unless (ref $fmt eq 'CODE') {
     #    my $x = $fmt;
     #    $fmt = sub { sprintf $x, shift };
@@ -873,14 +637,8 @@ Refreshes stateful information like the universal set, if present.
 
 sub refresh {
     my $self = shift;
-    if (my $u = $self->_universe) {
-        my $univ = $u->();
-        if (my $t = $self->composite) {
-            if (my $c = $t->coercion) {
-                $univ = $c->coerce($univ);
-            }
-        }
-        $self->_unicache($univ);
+    if (my $u = $self->universe) {
+        $self->_unicache($u->());
     }
 
     1;
@@ -892,18 +650,8 @@ Dorian Taylor, C<< <dorian at cpan.org> >>
 
 =head1 SEE ALSO
 
-=over 4
-
-=item
-
 L<Params::Registry>
 
-=item
-
-L<Params::Registry::Instance>
-
-=back
-
 =head1 LICENSE AND COPYRIGHT
 
 Copyright 2013 Dorian Taylor.
